from django.conf import settings
from django.urls import include, path

from galaxy_ng.app.api import urls as api_urls
from galaxy_ng.app import customadmin as admin
from galaxy_ng.ui import urls as ui_urls


API_PATH_PREFIX = settings.GALAXY_API_PATH_PREFIX.strip("/")

galaxy_urls = [
    path(f"{API_PATH_PREFIX}/", include(api_urls)),
]

urlpatterns = [
    path("", include((galaxy_urls, "api"), namespace="galaxy")),
    path("", include(ui_urls)),
    path("", include("django_prometheus.urls")),
    path(settings.ADMIN_SITE_URL, admin.site.urls),
]



from django.urls import path, include
from rest_framework import routers

from galaxy_ng.app import constants

from . import views
from . import viewsets


router = routers.SimpleRouter()
# TODO: Replace with a RedirectView
router.register('namespaces', viewsets.NamespaceViewSet, basename='namespaces')
router.register('my-namespaces', viewsets.MyNamespaceViewSet, basename='my-namespaces')
router.register('my-synclists', viewsets.MySyncListViewSet, basename='my-synclists')
router.register('users', viewsets.UserViewSet, basename='users')
router.register('collection-versions',
                viewsets.CollectionVersionViewSet, basename='collection-versions')
router.register(
    'imports/collections',
    viewsets.CollectionImportViewSet,
    basename='collection-imports',
)
router.register('tags', viewsets.TagsViewSet, basename='tags')
router.register('synclists', viewsets.SyncListViewSet, basename='synclists')
router.register('remotes', viewsets.CollectionRemoteViewSet, basename='remotes')
router.register('distributions', viewsets.DistributionViewSet, basename='distributions')
router.register('my-distributions', viewsets.MyDistributionViewSet, basename='my-distributions')

auth_views = [
    path("login/", views.LoginView.as_view(), name="auth-login"),
    path("logout/", views.LogoutView.as_view(), name="auth-logout"),
]

# Groups are subclassed from pulpcore and use nested viewsets, so router.register
# unfortunately doesn't work
group_paths = [
    path(
        "",
        viewsets.GroupViewSet.as_view({'get': 'list', 'post': 'create'}),
        name='groups'),
    path(
        "<str:pk>/",
        viewsets.GroupViewSet.as_view({'get': 'retrieve', 'delete': 'destroy'}),
        name='group-detail'),
    path(
        "<str:group_pk>/model-permissions/",
        viewsets.GroupModelPermissionViewSet.as_view({
            'get': 'list', 'post': 'create'}),
        name='group-model-permissions'),
    path(
        "<str:group_pk>/model-permissions/<str:pk>/",
        viewsets.GroupModelPermissionViewSet.as_view({
            'get': 'retrieve', 'delete': 'destroy'}),
        name='group-model-permissions-detail'),
    path(
        "<str:group_pk>/users/",
        viewsets.GroupUserViewSet.as_view({
            'get': 'list', 'post': 'create'}),
        name='group-model-permissions'),
    path(
        "<str:group_pk>/users/<str:pk>/",
        viewsets.GroupUserViewSet.as_view({
            'delete': 'destroy'}),
        name='group-model-permissions-detail'),
]

paths = [
    path('', include(router.urls)),

    path('auth/', include(auth_views)),
    path('groups/', include(group_paths)),

    path(
        'repo/<str:path>/',
        viewsets.CollectionViewSet.as_view({'get': 'list'}),
        name='collections-list'
    ),
    path(
        'repo/<str:path>/<str:namespace>/<str:name>/',
        viewsets.CollectionViewSet.as_view({'get': 'retrieve'}),
        name='collections-detail'
    ),

    path(
        'collections/',
        viewsets.CollectionViewSetDeprecated.as_view({'get': 'list'}),
        name='collections-list'
    ),
    path(
        'collections/<str:namespace>/<str:name>/',
        viewsets.CollectionViewSetDeprecated.as_view({'get': 'retrieve'}),
        name='collections-detail'
    ),

    # NOTE: Using path instead of SimpleRouter because SimpleRouter expects retrieve
    # to look up values with an ID
    path(
        'me/',
        viewsets.CurrentUserViewSet.as_view({'get': 'retrieve', 'put': 'update'}),
        name='me'
    )

]
app_name = "ui"

urlpatterns = [
    path('', viewsets.APIRootView.as_view({'get': 'list'}))
]

for version in constants.ALL_UI_API_VERSION:
    urlpatterns.append(path(
        constants.ALL_UI_API_VERSION[version],
        include((paths, app_name), namespace=version)
    ))



from .auth import (
    LoginSerializer,
)
from .collection import (
    CollectionDetailSerializer,
    CollectionListSerializer,
    CollectionVersionSerializer,
    CollectionVersionDetailSerializer,
    CollectionVersionBaseSerializer,
)
from .imports import (
    ImportTaskDetailSerializer,
    ImportTaskListSerializer,
)

from .user import (
    UserSerializer,
    CurrentUserSerializer,
)

from .synclist import (
    SyncListSerializer,
    SyncListCollectionSummarySerializer,
)

from .distribution import (
    DistributionSerializer
)

__all__ = (
    # auth
    'LoginSerializer',
    # collection
    'CollectionDetailSerializer',
    'CollectionListSerializer',
    'CollectionVersionSerializer',
    'CollectionVersionDetailSerializer',
    'CollectionVersionBaseSerializer',
    # imports
    'ImportTaskDetailSerializer',
    'ImportTaskListSerializer',
    # current_user
    'CurrentUserSerializer',
    # user
    'UserSerializer',
    # synclist
    'SyncListSerializer',
    'SyncListCollectionSummarySerializer',
    # distribution,
    'DistributionSerializer'
)



from rest_framework import serializers

from pulp_ansible.app import viewsets as pulp_viewsets
from pulp_ansible.app.models import (
    AnsibleDistribution,
    AnsibleRepository,
    CollectionRemote,
)

from galaxy_ng.app.constants import COMMUNITY_DOMAINS
from galaxy_ng.app.models.collectionsync import CollectionSyncTask
from galaxy_ng.app.common.proxy_url import strip_auth_from_url, join_proxy_url


class AnsibleDistributionSerializer(serializers.ModelSerializer):
    created_at = serializers.DateTimeField(source='pulp_created')
    updated_at = serializers.DateTimeField(source='pulp_last_updated')

    class Meta:
        model = AnsibleDistribution
        fields = (
            'name',
            'base_path',
            'content_guard',
            'created_at',
            'updated_at',
        )


class LastSyncTaskMixin:

    def get_last_sync_task_queryset(self, obj):
        raise NotImplementedError("subclass must implement get_last_sync_task_queryset")

    def get_last_sync_task(self, obj):
        sync_task = self.get_last_sync_task_queryset(obj)
        if not sync_task:
            # UI handles `null` as "no status"
            return

        return {
            "task_id": sync_task.id,
            "state": sync_task.task.state,
            "started_at": sync_task.task.started_at,
            "finished_at": sync_task.task.finished_at,
            "error": sync_task.task.error
        }


class AnsibleRepositorySerializer(LastSyncTaskMixin, serializers.ModelSerializer):
    distributions = serializers.SerializerMethodField()
    last_sync_task = serializers.SerializerMethodField()
    created_at = serializers.DateTimeField(source='pulp_created')
    updated_at = serializers.DateTimeField(source='pulp_last_updated')

    class Meta:
        model = AnsibleRepository
        fields = (
            'name',
            'description',
            'next_version',
            'distributions',
            'created_at',
            'updated_at',
            'last_sync_task',
        )

    def get_distributions(self, obj):
        return [
            AnsibleDistributionSerializer(distro).data
            for distro in obj.ansible_ansibledistribution.all()
        ]

    def get_last_sync_task_queryset(self, obj):
        return CollectionSyncTask.objects.order_by(
            '-task__pulp_last_updated').filter(repository=obj).first()


class CollectionRemoteSerializer(LastSyncTaskMixin, pulp_viewsets.CollectionRemoteSerializer):
    last_sync_task = serializers.SerializerMethodField()
    write_only_fields = serializers.SerializerMethodField()
    created_at = serializers.DateTimeField(source='pulp_created', required=False)
    updated_at = serializers.DateTimeField(source='pulp_last_updated', required=False)
    proxy_url = serializers.URLField(
        help_text="The proxy url e.g: http://IP:PORT",
        allow_null=True,
        required=False,
    )
    proxy_username = serializers.CharField(
        allow_null=True,
        required=False
    )
    proxy_password = serializers.CharField(
        allow_null=True,
        required=False,
        style={'input_type': 'password'},
        write_only=True
    )
    token = serializers.CharField(
        allow_null=True,
        required=False,
        max_length=2000,
        write_only=True,
        style={'input_type': 'password'}
    )
    password = serializers.CharField(
        help_text="The password to be used for authentication when syncing.",
        allow_null=True,
        required=False,
        style={'input_type': 'password'},
        write_only=True
    )
    name = serializers.CharField(read_only=True)
    repositories = serializers.SerializerMethodField()

    class Meta:
        model = CollectionRemote
        fields = (
            'pk',
            'name',
            'url',
            'auth_url',
            'token',
            'policy',
            'requirements_file',
            'created_at',
            'updated_at',
            'username',
            'password',
            'tls_validation',
            'client_key',
            'client_cert',
            'ca_cert',
            'last_sync_task',
            'repositories',
            'pulp_href',
            'download_concurrency',
            'proxy_url',
            'proxy_username',
            'proxy_password',
            'write_only_fields'
        )
        extra_kwargs = {
            'name': {'read_only': True},
            'pulp_href': {'read_only': True},
            'client_key': {'write_only': True},
        }

    def to_representation(self, instance):
        """
        Splits proxy_url field from DB in 3 fields for representation.
        proxy_url, proxy_username, proxy_password (write-only)
        """

        data = super().to_representation(instance)

        if instance.proxy_url is not None:
            url, username, _ = strip_auth_from_url(instance.proxy_url)
            data['proxy_url'] = url
            data['proxy_username'] = username

        return data

    def save(self):
        """
        Rejoins 3 fields url, username, password back to proxy_url on DB.

        If proxy_url is null or blank, the field is cleaned up in DB
        regardless of username, password values.

        If proxy, url, password or username is missing from the payload then
        should be the same as the existing value in the DB or None.

        If proxy username/password is set to null or blank, the fields are
        cleaned up in DB.
        """

        empty = object()
        proxy_url = self.validated_data.get('proxy_url', empty)
        proxy_username = self.validated_data.get('proxy_username', empty)
        proxy_password = self.validated_data.get('proxy_password', empty)

        url_in_db, username_in_db, password_in_db = strip_auth_from_url(
            self.instance.proxy_url
        ) if self.instance.proxy_url else (None, None, None)

        if proxy_url is empty:
            proxy_url = url_in_db

        if proxy_username is empty:
            proxy_username = username_in_db

        if proxy_password is empty:
            proxy_password = password_in_db

        self.validated_data['proxy_url'] = join_proxy_url(
            proxy_url,
            proxy_username,
            proxy_password,
        ) if proxy_url else None

        super().save()

    def validate(self, data):
        if not data.get('requirements_file') and any(
            [domain in data['url'] for domain in COMMUNITY_DOMAINS]
        ):
            raise serializers.ValidationError(
                detail={
                    'requirements_file':
                        'Syncing content from community domains without specifying a '
                        'requirements file is not allowed.'
                }
            )
        return super().validate(data)

    def get_write_only_fields(self, obj):
        url = obj.proxy_url
        proxy_password = None
        if url is not None:
            proxy_password = strip_auth_from_url(url)[2]
        return get_write_only_fields(self, obj, extra_data={'proxy_password': proxy_password})

    def get_repositories(self, obj):
        return [
            AnsibleRepositorySerializer(repo).data
            for repo in obj.repository_set.all()
        ]

    def get_last_sync_task_queryset(self, obj):
        """Gets last_sync_task from Pulp using remote->repository relation"""

        return CollectionSyncTask.objects.order_by(
            '-task__pulp_last_updated').filter(
            repository=obj.repository_set.order_by('pulp_last_updated').first()
        ).first()


def get_write_only_fields(serializer, obj, extra_data={}):
    """
    Returns a list of write only fields and whether or not their values are set
    so that clients can tell if they are overwriting an existing value.
    serializer: Serializer instance
    obj: model object being serialized
    extra_data: extra fields that might not be on obj. This is used when a write
        only field is not one of the fields in the underlying data model.
    """
    fields = []

    # returns false if field is "" or None
    def _is_set(field_name):
        if (field_name in extra_data):
            return bool(extra_data[field_name])
        else:
            return bool(getattr(obj, field_name))

    # There are two ways to set write_only. This checks both.

    # check for values that are set to write_only in Meta.extra_kwargs
    for field_name in serializer.Meta.extra_kwargs:
        if serializer.Meta.extra_kwargs[field_name].get('write_only', False):
            fields.append({"name": field_name, "is_set": _is_set(field_name)})

    # check for values that are set to write_only in fields
    serializer_fields = serializer.get_fields()
    for field_name in serializer_fields:
        if (serializer_fields[field_name].write_only):
            fields.append({"name": field_name, "is_set": _is_set(field_name)})

    return fields



from collections import namedtuple
import re


CollectionFilename = namedtuple("CollectionFilename", ["namespace", "name", "version"])

FILENAME_REGEXP = re.compile(
    r"^(?P<namespace>\w+)-(?P<name>\w+)-" r"(?P<version>[0-9a-zA-Z.+-]+)\.tar\.gz$"
)
VERSION_REGEXP = re.compile(r"""
^
(?P<major>0|[1-9][0-9]*)
\.
(?P<minor>0|[1-9][0-9]*)
\.
(?P<patch>0|[1-9][0-9]*)
(?:
    -(?P<pre>[0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*)
)?
(?:
    \+(?P<build>[0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*)
)?
$
""", re.VERBOSE | re.ASCII)


def parse_collection_filename(filename):
    """
    Parses collection filename.

    Parses and validates collection filename. Returns CollectionFilename named tuple.
    Raises ValueError if filename is not a valid collection filename.
    """
    match = FILENAME_REGEXP.match(filename)

    if not match:
        msg = "Invalid filename {filename}. Expected format: namespace-name-version.tar.gz"
        raise ValueError(msg.format(filename=filename))

    namespace, name, version = match.groups()

    match = VERSION_REGEXP.match(version)
    if not match:
        msg = "Invalid version string {version} from filename {filename}. Expected semantic version format." # noqa
        raise ValueError(msg.format(version=version, filename=filename))

    return CollectionFilename(namespace, name, version)



"""
This module is temporarily added to address the
issue AAH-291.

The change that this module is solving
is going to be implemented on pulpcore
and then this module and its references
can be removed or replaces with pulpcore
implementation.
"""
from typing import List, Tuple, Optional
from urllib3.util import parse_url, Url


def get_parsed_url(parsed: Url, auth_items: Optional[List[Optional[str]]]) -> str:
    """
    parsed: Url instance
    auth_items: Optional list of str ["user": "password"]
    returns: str e.g: http://user:pass@foo.bar:9999
    """
    return Url(
        scheme=parsed.scheme,
        auth=auth_items and ":".join(auth_items) or None,
        host=parsed.host,
        port=parsed.port,
        path=parsed.path,
        query=parsed.query,
        fragment=parsed.fragment,
    ).url


def strip_auth_from_url(url: str) -> Tuple[str, Optional[str], Optional[str]]:
    """
    Gets a full proxy address and strips its auth
    # url: http://bruno:1234@foo.bar:9999/zaz/traz/?a#e;w
    # returns: tuple, e.g: "http://foo.bar:9999/zaz/traz/?a#e;w", bruno, 1234
    """

    if not url.startswith(("http://", "https://")):
        url = "http://" + url

    parsed = parse_url(url)
    auth = parsed.auth and parsed.auth.split(":")
    username = auth and auth[0] or None
    password = auth and len(auth) > 1 and auth[1] or None
    return (
        get_parsed_url(parsed, None),
        username,
        password,
    )


def join_proxy_url(address: str, username: Optional[str], password: Optional[str]) -> str:
    """
    Gets a splitted address, username, password
    returns: Joined URL forming proxy_url
    """
    # http://username:password@address

    if not address.startswith(("http://", "https://")):
        address = "http://" + address

    parsed = parse_url(address)
    auth_items = []

    if username:
        auth_items.append(str(username))

        # password is set only if there is a username
        # to avoid it being set as e.g: http://:1234@foo.bar
        if password:
            auth_items.append(str(password))

    return get_parsed_url(parsed, auth_items)



import logging

from rest_framework import serializers

from galaxy_ng.app.api.ui.serializers.base import Serializer

log = logging.getLogger(__name__)


class TaskSerializer(Serializer):
    pulp_id = serializers.UUIDField(source='pk')
    name = serializers.CharField()
    state = serializers.CharField()
    started_at = serializers.DateTimeField()
    finished_at = serializers.DateTimeField()



import logging

from django.conf import settings
from rest_access_policy import AccessPolicy
from rest_framework.exceptions import NotFound

from galaxy_ng.app import models

from galaxy_ng.app.access_control.statements import STANDALONE_STATEMENTS, INSIGHTS_STATEMENTS

log = logging.getLogger(__name__)

STATEMENTS = {'insights': INSIGHTS_STATEMENTS,
              'standalone': STANDALONE_STATEMENTS}


class AccessPolicyBase(AccessPolicy):
    def _get_statements(self, deployment_mode):
        return STATEMENTS[deployment_mode]

    def get_policy_statements(self, request, view):
        statements = self._get_statements(settings.GALAXY_DEPLOYMENT_MODE)
        return statements.get(self.NAME, [])

    # used by insights access policy
    def has_rh_entitlements(self, request, view, permission):
        if not isinstance(request.auth, dict):
            return False
        header = request.auth.get('rh_identity')
        if not header:
            return False
        entitlements = header.get('entitlements', {})
        entitlement = entitlements.get(settings.RH_ENTITLEMENT_REQUIRED, {})
        return entitlement.get('is_entitled', False)


class NamespaceAccessPolicy(AccessPolicyBase):
    NAME = 'NamespaceViewSet'


class CollectionAccessPolicy(AccessPolicyBase):
    NAME = 'CollectionViewSet'

    def can_update_collection(self, request, view, permission):
        collection = view.get_object()
        namespace = models.Namespace.objects.get(name=collection.namespace)
        return request.user.has_perm('galaxy.upload_to_namespace', namespace)

    def can_create_collection(self, request, view, permission):
        data = view._get_data(request)
        try:
            namespace = models.Namespace.objects.get(name=data['filename'].namespace)
        except models.Namespace.DoesNotExist:
            raise NotFound('Namespace in filename not found.')
        return request.user.has_perm('galaxy.upload_to_namespace', namespace)


class CollectionRemoteAccessPolicy(AccessPolicyBase):
    NAME = 'CollectionRemoteViewSet'


class UserAccessPolicy(AccessPolicyBase):
    NAME = 'UserViewSet'

    def user_is_superuser(self, request, view, action):
        user = view.get_object()
        return user.is_superuser

    def is_current_user(self, request, view, action):
        return request.user == view.get_object()


class MyUserAccessPolicy(AccessPolicyBase):
    NAME = 'MyUserViewSet'

    def is_current_user(self, request, view, action):
        return request.user == view.get_object()


class SyncListAccessPolicy(AccessPolicyBase):
    NAME = 'SyncListViewSet'


class MySyncListAccessPolicy(AccessPolicyBase):
    NAME = 'MySyncListViewSet'


class TagsAccessPolicy(AccessPolicyBase):
    NAME = 'TagViewSet'


class TaskAccessPolicy(AccessPolicyBase):
    NAME = 'TaskViewSet'


class LoginAccessPolicy(AccessPolicyBase):
    NAME = 'LoginView'


class LogoutAccessPolicy(AccessPolicyBase):
    NAME = 'LogoutView'


class TokenAccessPolicy(AccessPolicyBase):
    NAME = 'TokenView'


class GroupAccessPolicy(AccessPolicyBase):
    NAME = 'GroupViewSet'


class DistributionAccessPolicy(AccessPolicyBase):
    NAME = 'DistributionViewSet'


class MyDistributionAccessPolicy(AccessPolicyBase):
    NAME = 'MyDistributionViewSet'



#!/usr/bin/env python3

import os
import tempfile
import tarfile
import urllib.request
from distutils import log

from setuptools import find_packages, setup, Command
from setuptools.command.build_py import build_py as _BuildPyCommand
from setuptools.command.sdist import sdist as _SDistCommand

from galaxy_ng import __version__


class PrepareStaticCommand(Command):
    UI_DOWNLOAD_URL = (
        "https://github.com/ansible/ansible-hub-ui/releases/download/"
        f"{__version__}/automation-hub-ui-dist.tar.gz"
    )
    TARGET_DIR = "galaxy_ng/app/static/galaxy_ng"

    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if os.path.exists(self.TARGET_DIR):
            log.warn(f"Static directory {self.TARGET_DIR} already exists, skipping. ")
            return

        with tempfile.NamedTemporaryFile() as download_file:
            log.info(f"Downloading UI distribution to temporary file: {download_file.name}")
            urllib.request.urlretrieve(self.UI_DOWNLOAD_URL, filename=download_file.name)

            log.info(f"Extracting UI static files to {self.TARGET_DIR}")
            with tarfile.open(fileobj=download_file) as tfp:
                tfp.extractall(self.TARGET_DIR)


class SDistCommand(_SDistCommand):
    def run(self):
        self.run_command('prepare_static')
        return super().run()


class BuildPyCommand(_BuildPyCommand):
    def run(self):
        self.run_command('prepare_static')
        return super().run()


requirements = [
    "Django~=2.2.20",
    "galaxy-importer==0.2.15",
    "pulpcore>=3.7,<3.9",
    "pulp-ansible==0.5.8",
    "django-prometheus>=2.0.0",
    "drf-spectacular",
]

package_name = os.environ.get("GALAXY_NG_ALTERNATE_NAME", "galaxy-ng")

setup(
    name=package_name,
    version=__version__,
    description="galaxy-ng plugin for the Pulp Project",
    license="GPLv2+",
    author="Red Hat, Inc.",
    author_email="info@ansible.com",
    url="https://github.com/ansible/galaxy_ng/",
    python_requires=">=3.6",
    setup_requires=["wheel"],
    install_requires=requirements,
    include_package_data=True,
    packages=find_packages(exclude=["tests", "tests.*"]),
    classifiers=(
        "License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)",
        "Operating System :: POSIX :: Linux",
        "Framework :: Django",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ),
    entry_points={"pulpcore.plugin": ["galaxy_ng = galaxy_ng:default_app_config"]},
    cmdclass={
        "prepare_static": PrepareStaticCommand,
        "build_py": BuildPyCommand,
        "sdist": SDistCommand,
    },
)



import logging

from pulpcore.plugin import viewsets as pulp_core_viewsets

from galaxy_ng.app.api.base import LocalSettingsMixin
from galaxy_ng.app.api.v3.serializers import TaskSerializer
from galaxy_ng.app.access_control import access_policy

log = logging.getLogger(__name__)


class TaskViewSet(LocalSettingsMixin, pulp_core_viewsets.TaskViewSet):
    permission_classes = [access_policy.TaskAccessPolicy]
    serializer_class = TaskSerializer



from django.urls import path, include
from rest_framework import routers

from . import views
from . import viewsets

# TODO: remove this file once the UI has been updated to use the new versioned URLs

router = routers.SimpleRouter()
# TODO: Replace with a RedirectView
router.register('namespaces', viewsets.NamespaceViewSet, basename='namespaces')
router.register('my-namespaces', viewsets.MyNamespaceViewSet, basename='my-namespaces')
router.register('users', viewsets.UserViewSet, basename='users')
router.register('collection-versions',
                viewsets.CollectionVersionViewSet, basename='collection-versions')
router.register(
    'imports/collections',
    viewsets.CollectionImportViewSet,
    basename='collection-imports',
)
router.register('tags', viewsets.TagsViewSet, basename='tags')
router.register('synclists', viewsets.SyncListViewSet, basename='synclists')

auth_views = [
    path("login/", views.LoginView.as_view(), name="auth-login"),
    path("logout/", views.LogoutView.as_view(), name="auth-logout"),
]

app_name = "old_ui"
urlpatterns = [
    path('', include(router.urls)),

    path('auth/', include(auth_views)),

    path(
        'collections/',
        viewsets.CollectionViewSetDeprecated.as_view({'get': 'list'}),
        name='collections-list'
    ),
    path(
        'collections/<str:namespace>/<str:name>/',
        viewsets.CollectionViewSetDeprecated.as_view({'get': 'retrieve'}),
        name='collections-detail'
    ),

    # NOTE: Using path instead of SimpleRouter because SimpleRouter expects retrieve
    # to look up values with an ID
    path(
        'me/',
        viewsets.CurrentUserViewSet.as_view({'get': 'retrieve', 'put': 'update'}),
        name='me'
    )
]
